@@ -1,19 +1,31 @@
 __title__ = 'datclass'
 __author__ = 'foyoux'
-__version__ = '0.0.11'
-__all__ = ['main', 'DatClass', 'List', 'Dict', 'Union', 'get_origin', 'get_args', 'dataclass', 'field', 'is_dataclass']
+__version__ = '0.0.13'
+__all__ = [
+    'main',
+    'DatClass',
+    'List',
+    'Dict',
+    'Union',
+    'TypedDict',
+    'get_origin',
+    'get_args',
+    'dataclass',
+    'field',
+    'is_dataclass'
+]
 
 import argparse
 import json
 import os
-from dataclasses import dataclass, is_dataclass, field
+from dataclasses import dataclass, is_dataclass, field, asdict
 from pathlib import Path
 from typing import List, Dict, Union
 
 try:
-    from typing import get_origin, get_args
+    from typing import get_origin, get_args, TypedDict
 except ImportError:
-    from typing_extensions import get_origin, get_args
+    from typing_extensions import get_origin, get_args, TypedDict
 
 _ORIGINAL_INIT = '__dataclass_init__'
 
@@ -37,8 +49,8 @@ class DatClass:
         return obj
 
     def __post_init__(self, *args, **kwargs):
-        for attr_name, field in self.__dataclass_fields__.items():  # type: ignore
-            attr_type = field.type
+        for attr_name, FIELD in self.__dataclass_fields__.items():  # type: ignore
+            attr_type = FIELD.type
             origin = get_origin(attr_type)
             if origin is None and is_dataclass(attr_type):
                 setattr(self, attr_name, attr_type(**getattr(self, attr_name)))
@@ -85,7 +97,7 @@ def get_t_string(t):
 
 
 def get_nice_cls_name(field_name: str):
-    return field_name.title()
+    return field_name.title().replace('_', '')
 
 
 def merge_list_dict(list_dict: List[dict]) -> Dict:
@@ -103,47 +115,105 @@ def merge_list_dict(list_dict: List[dict]) -> Dict:
     return d
 
 
-imports = [
-    'from datclass import dataclass, field, List, Dict, DatClass', '', '',
-]
+@dataclass
+class Imports:
+    dataclass: bool = False
+    field: bool = False
+    List: bool = False
+    Dict: bool = False
+    TypedDict: bool = False
+    DatClass: bool = False
+
+    def to_list(self):
+        return [f'from datclass import {", ".join([k for k, v in asdict(self).items() if v])}', '', '']
 
 
-def gen_datclass(dat: Union[list, dict], name='Object', recursive=False):
+imports = Imports()
+
+
+def gen_datclass(dat: Union[list, dict], name='Object', recursive=False, dict_=False):
     """
     :param dat: list or dict data
     :param name: main dat class name
-    :param recursive: recursive generate dat class
+    :param recursive: recursive generate datclass
+    :param dict_: generate TypedDict class
     """
     try:
         dat = merge_list_dict(dat)
     except TypeError:
         pass
-    codes = ['@dataclass', f'class {name}(DatClass):']
+
+    if dict_:
+        imports.TypedDict = True
+        codes = [f'class {name}(TypedDict):']
+    else:
+        imports.dataclass = True
+        imports.DatClass = True
+        codes = ['@dataclass', f'class {name}(DatClass):']
 
     for k, v in dat.items():
         v_t = get_v_type(v)
         v_d = get_t_default(v_t)
         t_s = get_t_string(v_t)
+        if recursive and v and (isinstance(v, dict) or t_s == 'List[dict]'):
+            imports.List = True
+            s = get_nice_cls_name(k)
+            if isinstance(v, dict):
+                t_s = s
+            elif t_s == 'List[dict]':
+                t_s = f'List[{s}]'
+            codes = gen_datclass(v, s, recursive=True, dict_=dict_) + ['', ''] + codes
+        if t_s == 'Dict':
+            imports.Dict = True
+        if dict_:
+            codes.append(f'    {k}: {t_s}')
+        else:
+            if v_d.startswith('field'):
+                imports.field = True
+            codes.append(f'    {k}: {t_s} = {v_d}')
+
+    return codes
+
+
+def gen_typed_dict(dat: Union[list, dict], name='Object', recursive=False):
+    try:
+        dat = merge_list_dict(dat)
+    except TypeError:
+        pass
+    imports.TypedDict = True
+    codes = []
+    n_t_dict = {}
+    for k, v in dat.items():
+        v_t = get_v_type(v)
+        t_s = get_t_string(v_t)
         if recursive and v and (isinstance(v, dict) or t_s == 'List[dict]'):
             s = get_nice_cls_name(k)
             if isinstance(v, dict):
                 t_s = s
             elif t_s == 'List[dict]':
                 t_s = f'List[{s}]'
-            codes = gen_datclass(v, s, recursive=True) + ['', ''] + codes
-        codes.append(f'    {k}: {t_s} = {v_d}')
-
+            codes = gen_typed_dict(v, s, recursive=True) + codes
+        if not imports.Dict and t_s == 'Dict':
+            imports.Dict = True
+        if not imports.List and t_s.startswith('List'):
+            imports.List = True
+        n_t_dict[k] = t_s
+    s = ', '.join([f'\'{k}\': {v}' for k, v in n_t_dict.items()])
+    codes.append(f'{name} = TypedDict(\'{name}\', {{{s}}})')
     return codes
 
 
 def main():
     epilog = f'%(prog)s({__version__}) by foyoux(https://github.com/foyoux/datclass)'
-    parser = argparse.ArgumentParser(prog='datclass', description='', epilog=epilog)
+    parser = argparse.ArgumentParser(prog='datclass', description='generate datclass & support nested and extra',
+                                     epilog=epilog)
     parser.add_argument('-v', '--version', action='version', version=epilog)
 
     parser.add_argument('-n', '--name', help='main dat class name', default='Object')
     parser.add_argument('-r', '--recursive', help='recursive generate dat class', action='store_true')
     parser.add_argument('-o', '--output', help='output file - *.py')
+    parser.add_argument('-d', '--dict', help='generate TypedDict class', action='store_true')
+    parser.add_argument('-i', '--inline', help='use inline model to generate TypedDict type', action='store_true')
     parser.add_argument('file', nargs='?', help='input file - likes-json')
 
     args = parser.parse_args()
@@ -177,8 +247,12 @@ def main():
         print('\nInvalid JSON data')
         return
 
-    dat = gen_datclass(body, name, recursive)
-    dat = '\n'.join(imports + dat + [''])
+    if args.dict and args.inline:
+        dat = gen_typed_dict(body, name, recursive)
+    else:
+        dat = gen_datclass(body, name, recursive, args.dict)
+
+    dat = '\n'.join(imports.to_list() + dat + [''])
 
     if output:
         f = Path(output)
